# require 'spec_helper'
#
# describe JIRA::Resource::Issuelink do
#   let(:client) { double() }
#
#   describe "links" do
#     subject {
#       JIRA::Resource::Issuelink.new(client, :attrs => {
#
#       }
#       )
#     }
#   end
# end
